/***********************************************************************************************************************
 *
 * blueBill Mobile - open source birdwatching
 * ==========================================
 *
 * Copyright (C) 2009, 2010 by Tidalwave s.a.s. (http://www.tidalwave.it)
 * http://bluebill.tidalwave.it/mobile/
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: TaxonSoundFactSheetController.java,v e478eae6271a 2010/06/27 01:07:27 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.mobile.taxonomy.factsheet.sound;

import java.io.UnsupportedEncodingException;
import javax.annotation.CheckForNull;
import javax.annotation.Nonnegative;
import javax.annotation.Nonnull;
import javax.inject.Provider;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Arrays;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.IOException;
import java.io.Reader;
import org.openide.util.Exceptions;
import org.openide.util.NbBundle;
import it.tidalwave.util.AsException;
import it.tidalwave.util.Identifiable;
import it.tidalwave.util.logging.Logger;
import it.tidalwave.netbeans.util.Locator;
import it.tidalwave.mobile.util.Downloadable;
import it.tidalwave.mobile.util.Downloadable.Status;
import it.tidalwave.mobile.media.MediaPlayer;
import it.tidalwave.mobile.media.MediaPlayer.Controller;
import it.tidalwave.mobile.media.Media;
import it.tidalwave.bluebill.observation.Observation;
import it.tidalwave.bluebill.taxonomy.Taxon;
import it.tidalwave.bluebill.observation.ObservationSet;
import it.tidalwave.bluebill.mobile.network.NetworkingPreferences;
import it.tidalwave.mobile.io.FileSystem;
import it.tidalwave.semantic.io.json.RdfJsonUnmarshaller;
import it.tidalwave.util.Removable;
import java.net.URLEncoder;
import static it.tidalwave.mobile.util.Downloadable.Status.*;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: $
 *
 **********************************************************************************************************************/
public abstract class TaxonSoundFactSheetController
  {
    private static final String CLASS = TaxonSoundFactSheetController.class.getName();
    private static final Logger logger = Logger.getLogger(CLASS);

    private final static double MEGA = 1024.0 * 1024.0;
    /* package */ final static long MIN_FREE_SPACE = 20L * 1024 * 1024;

    private enum Operation
      {
        DOWNLOAD
          {
            public void run (final @Nonnull Downloadable downloadable)
              {
                downloadable.download();
              }
          },
        REFRESH
          {
            public void run (final @Nonnull Downloadable downloadable)
              {
                downloadable.refresh();
              }
          };

        public abstract void run (@Nonnull Downloadable downloadable);
      }
    @Nonnull
    private final TaxonSoundFactSheetUI ui;

    @Nonnull
    protected final Taxon taxon;

    @CheckForNull
    /* package */ Controller controller;

    @Nonnull
    private final Provider<MediaPlayer> mediaPlayer = Locator.createProviderFor(MediaPlayer.class);

    @Nonnull
    private final Provider<FileSystem> fileSystem = Locator.createProviderFor(FileSystem.class);

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    /* package */ final PropertyChangeListener downloadablePropertyChangeListener = new PropertyChangeListener()
      {
        public void propertyChange (final @Nonnull PropertyChangeEvent event)
          {
//            logger.info("propertyChange(%s)", event);

            if ((event.getPropertyName().equals(Downloadable.PROP_STATUS)) &&
                (event.getOldValue() != event.getNewValue()))
              {
                switch ((Status)event.getNewValue())
                  {
                    case DOWNLOADED:
                      final double freeSpace = fileSystem.get().getFreeSpace() / MEGA;
                      ui.notify(NbBundle.getMessage(TaxonSoundFactSheetController.class, "downloadComplete", freeSpace));
                      ((Downloadable)event.getSource()).removePropertyChangeListener(downloadablePropertyChangeListener);
                      break;

                    case BROKEN:
                      ui.notify(NbBundle.getMessage(TaxonSoundFactSheetController.class, "downloadFailed"));
                      ((Downloadable)event.getSource()).removePropertyChangeListener(downloadablePropertyChangeListener);
                      break;

                    case DOWNLOADING:
                    case NOT_DOWNLOADED:
                    case OBSOLETE:
                    case QUEUED:
                  }
              }
          }
      };

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public TaxonSoundFactSheetController (final @Nonnull TaxonSoundFactSheetUI ui, final @Nonnull Taxon taxon)
      throws IOException
      {
        this.ui = ui;
        this.taxon = taxon;
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public void disposePlayer()
      {
        if (controller != null)
          {
            controller.dispose();
          }
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public boolean isDownloadable (final @Nonnegative int index)
      {
        return isStatus(index, NOT_DOWNLOADED, OBSOLETE, BROKEN);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public boolean isRefreshable (final @Nonnegative int index)
      {
        return isStatus(index, DOWNLOADED, OBSOLETE, BROKEN);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public boolean isDeletable (final @Nonnegative int index)
      {
        return isStatus(index, DOWNLOADED, OBSOLETE);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public boolean isPlayable (final @Nonnegative int index)
      {
        return isStatus(index, DOWNLOADED, OBSOLETE);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public boolean hasWebPage (final @Nonnegative int index)
      {
        try
          {
            getObservation(index).as(Identifiable.class).getId();
            return true;
          }
        catch (AsException e)
          {
            return false;
          }
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public void downloadMedia (final @Nonnegative int index)
      {
        if (isDownloadable(index))
          {
            downloadMediaWithConfirmation(index, Operation.DOWNLOAD);
          }
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public void refreshMedia (final @Nonnegative int index)
      {
        if (isRefreshable(index))
          {
            downloadMediaWithConfirmation(index, Operation.REFRESH);
          }
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public void deleteMedia (final @Nonnegative int index)
      {
        if (isDeletable(index))
          {
            final String title   = NbBundle.getMessage(TaxonSoundFactSheetController.class, "confirmDeletionTitle");
            final String message = NbBundle.getMessage(TaxonSoundFactSheetController.class, "confirmDeletionMessage");

            ui.getCommonUITasks().alertDialog(title, message, new Runnable()
              {
                public void run()
                  {
                    try
                      {
                        getMedia(index).as(Removable.class).remove();
                        final double freeSpace = fileSystem.get().getFreeSpace() / MEGA;
                        ui.notify(NbBundle.getMessage(TaxonSoundFactSheetController.class, "deletionCompleted", freeSpace));
                      }
                    catch (Exception e)
                      {
                        ui.notify(NbBundle.getMessage(TaxonSoundFactSheetController.class, "deletionFailed"));
                        logger.warning("Can't delete media %s", e);
                        logger.throwing("deleteMedia()", CLASS, e);
                      }
                  }
              });
          }
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public void playMedia (final @Nonnegative int index)
      {
        if (isPlayable(index))
          {
            try
              {
                disposePlayer();
                controller = mediaPlayer.get().play(getMedia(index));
                ui.notifyPlayMedia(controller);
              }
            catch (Exception e)
              {
                ui.notify(NbBundle.getMessage(TaxonSoundFactSheetController.class, "playFailed"));
                logger.warning("Can't play sound %s", e);
                logger.throwing("downloadMedia()", CLASS, e);
              }
          }
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public void goToPage (final @Nonnegative int index)
      {
        try
          {
            ui.openWebPage(getObservation(index).as(Identifiable.class).getId().stringValue());
          }
        catch (Exception e)
          {
            logger.warning("Can't open web page %s", e);
            logger.throwing("goToPage()", CLASS, e);
          }
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    protected Media getMedia (final @Nonnegative int index)
      {
        return getObservation(index).as(Media.class);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    protected abstract Observation getObservation (final @Nonnegative int index);

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    protected ObservationSet loadObservationSet (final @Nonnull InputStream is)
      throws IOException
      {
        final Reader r = new InputStreamReader(is, "UTF-8");
        final RdfJsonUnmarshaller unmarshaller = new RdfJsonUnmarshaller();
        return unmarshaller.unmarshal(r, ObservationSet.class);
      }
    
    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    private void downloadMediaWithConfirmation (final @Nonnegative int index, final @Nonnull Operation operation)
      {
        final String title   = NbBundle.getMessage(TaxonSoundFactSheetController.class, "confirmDownloadTitle");
        final String message = NbBundle.getMessage(TaxonSoundFactSheetController.class, "confirmDownloadMessage");

        if (Locator.find(NetworkingPreferences.class).isNetworkConnectionAllowed())
          {
            doDownloadMedia(index, operation);
          }
        else
          {
            ui.getCommonUITasks().alertDialog(title, message, new Runnable()
              {
                public void run()
                  {
                    doDownloadMedia(index, operation);
                  }
              });
          }
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    /* package */ void doDownloadMedia (final @Nonnegative int index, final @Nonnull Operation operation)
      {
        if (fileSystem.get().getFreeSpace() <= MIN_FREE_SPACE)
          {
            final String title   = NbBundle.getMessage(TaxonSoundFactSheetController.class, "downloadFailed", MIN_FREE_SPACE / MEGA);
            final String message = NbBundle.getMessage(TaxonSoundFactSheetController.class, "diskFullMessage", MIN_FREE_SPACE / MEGA);
            ui.getCommonUITasks().alertDialog(title, message, null);
          }
        else
          {
            final Downloadable downloadable = getMedia(index).as(Downloadable.class);
            downloadable.addPropertyChangeListener(downloadablePropertyChangeListener);
            operation.run(downloadable);
            ping();
          }
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    private boolean isStatus (final @Nonnegative int index, final @Nonnull Status ... status)
      {
        final Downloadable downloadable = getMedia(index).as(Downloadable.class);
        return Arrays.asList(status).contains(downloadable.getStatus());
      }

    protected void ping()
      {
      }
  }
